import importlib
import cv2
import numpy as np
import torch
from votTester.vot import VOTTester
from got10ktoolkit.got10k.trackers import Tracker
from got10ktoolkit.got10k.experiments import ExperimentGOT10k, ExperimentOTB, ExperimentLaSOT, ExperimentTrackingNet
import os

def get_parameters(tracker_name, param_file_name):
    param_module = importlib.import_module('pytracking.parameter.{}.{}'.format(tracker_name, param_file_name))
    params = param_module.parameters()
    return params

def read_image(image_file):
    return cv2.cvtColor(cv2.imread(image_file), cv2.COLOR_BGR2RGB)

class IdentityTracker(Tracker):
    def __init__(self, test_tracker):
        super(IdentityTracker, self).__init__(name='dmb_emd_tracker_lasot_epoch11')
        self.test_tracker = test_tracker

    def init(self, image, box):
        image_arr = np.array(image) # np[w, h, 3]
        self.box = box # np array [4]
        self.test_tracker.init(image_arr, box)

    def update(self, image):
        image_arr = np.array(image)
        self.box = self.test_tracker.update_got10K(image_arr)

        return self.box

if __name__ == '__main__':

####################################################################################
    tracker_module_dmb_train = importlib.import_module('pytracking.tracker.{}'.format('DMB_emd_tracker'))
    tracker_class_dmb_train = tracker_module_dmb_train.get_tracker_class()

    pth_path = os.path.join('/home/jaffe/PycharmProjects/DMB/pytracking/networks/logemd1_stm_track_dist_Noauthorinit_bz4/DAVIS16', 'recurrent11.pth.tar') #vos got10k:11
    #pth_path = '/home/jaffe/PycharmProjects/DMB/pytracking/networks/emd/recurrent1.pth.tar'
    param_module = importlib.import_module('pytracking.parameter.{}.{}'.format('DMB', 'DMB_emd_521_params'))
    params = param_module.parameters(pth_path)
    dmb_train_tracker = tracker_class_dmb_train(params)

    torch.backends.cudnn.benchmark = False
    tracker = IdentityTracker(dmb_train_tracker)

    # run experiments on GOT-10k (validation subset)
    result_dir = '/home/jaffe/PycharmProjects/DMB/got10ktoolkit/results'
    report_dir = '/home/jaffe/PycharmProjects/DMB/got10ktoolkit/results'

  #  experiment_got10k = ExperimentGOT10k('/home/jaffe/Titanspace/datasets/dataset/GOT-10k',result_dir=result_dir, subset='test')
  #  experiment_got10k.run(tracker, visualize=False)
    # report performance
  #  experiment_got10k.report([tracker.name])
    report_files = ['/home/jaffe/PycharmProjects/DMB/got10ktoolkit/results/performance_got10K.json']
    tracker_names = ['Ours', 'BACF', 'SiamFCv2', 'GOTURN', 'SiamFC',
                     'CCOT','ECO','CF2','MDNet','CFNetc5', 'CFNetc1',
                     'BACF','MEEM','DAT','DSST','SAMF','Staple','SRDCFdecon',
                     'SRDCF', 'fDSST', 'CSK', 'KCF', 'L1APG','LK','IVT']
    E = ExperimentGOT10k('/home/jaffe/Titanspace/datasets/dataset/GOT-10k', result_dir=result_dir, report_dir=report_dir, subset='test')
    E.plot_curves(report_files, tracker_names)



    # root_dir, version=2015, result_dir='results', report_dir='reports'
    #experiment_otb = ExperimentOTB(root_dir ='/home/jaffe/Titanspace/datasets/dataset/testdataset/dataset/OTB100', version=2015,\
    #                               result_dir=result_dir, report_dir=report_dir)
    #experiment_otb.run(tracker, visualize=False)
    #experiment_otb.report([tracker.name])


    # root_dir, subset='test', return_meta=False, result_dir='results', report_dir='reports'
   # experiment_lasot = ExperimentLaSOT(root_dir ='/home/jaffe/Titanspace/datasets/dataset/LaSOT/LaSOTBenchmark', subset='test',\
      #                                 result_dir=result_dir, report_dir=report_dir)
   # experiment_lasot.run(tracker, visualize=False)
  #  experiment_lasot.report([tracker.name])

    """"""
    # root_dir, subset='test', return_meta=False, result_dir='results', report_dir='reports'
    #experiment_tkn = ExperimentTrackingNet(root_dir ='/home/jaffe/Titanspace/datasets/dataset/TrackingNet', subset='test',\
    #                                         result_dir=result_dir, report_dir=report_dir)
    #experiment_tkn.run(tracker, visualize=False)
    #experiment_tkn.report([tracker.name], plot_curves=False)



